/**
 * Copyright (c) 2010 Tarkan Corak. All rights reserved.
 * Extended (2011) by David Kliczbor. 
 * Version: 2.0
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * Changelog (v2.0):
 * 		Added code for redirecting the user to the private page set by 
 * 			default.user.private.layout.friendly.url in portal-ext.properties
 * 		Introduced property (for portal-ext.properties): login.landing.page=<friendlyurlpattern>
 * 			Possible string replacements:
 * 				{ScreenName}				= screenname of current user
 * 				{UserURLGroup}				= user.getGroup().getFriendlyURL() (usually "/<screenname>")
 * 				{PrivateUserMapping}		= layout.friendly.url.private.user.servlet.mapping (default: "/user")
 * 				{PrivateGroupMapping}		= layout.friendly.url.private.group.servlet.mapping (default: "/group")
 * 				{PublicMapping}				= layout.friendly.url.public.servlet.mapping (default: "/web")
 * 				{UserDefaultPrivateLayout}	= default.user.private.layout.friendly.url (default: "/home") 
 * 				{UserDefaultPublicLayout}	= default.user.public.layout.friendly.url (default: "/home")
 * 				{UserGroupId:name}			= substituted by the ID of the UserGroup denoted by <name>
 * 				{any.property}				= will be replaced with the value of the property at time of login
 * 			Example:
 * 				login.landing.page={PrivateUserMapping}{UserURLGroup}{UserDefaultPrivateLayout}
 *	 				This example expands per default to: /user/screenname/home
 *				login.landing.page={PrivateGroupMapping}{UserURLGroup}/~/{UserGroupId:members}{UserDefaultPrivateLayout}
 *					This would expand to the URL of a user's private page that's being provided 
 *					by a Dynamic Site Template: /group/screenname/~/12345/home
 * 		Introduced property (for portal-ext.properties): login.landing.page.by.group
 * 			List of comma-separated "<usergroup>:<friendlyurlpattern>" tuples to redirect 
 * 			based on UserGroup memberships.
 * 			The same string replacements as for login.landing.page apply.
 * 			If the user belongs to a usergroup mentioned in this list, login.landing.page 
 * 			will be ignored and the corresponding <friendlyurlpattern> applies instead.
 * 			The _last_ applicable pattern will be applied!
 * 			Example:
 * 				login.landing.page.by.group=members:{PrivateGroupMapping}{UserURLGroup}/~/{UserGroupId:members}{UserDefaultPrivateLayout}, \
 * 											specialists:{PrivateGroupMapping}/special/home
 * 				In this example, a user that belongs to the groups "members" _and_ "specialists"
 * 				will be redirected to /group/special/home.
 * 				A user that's only in the group "members" will be redirected to his private home page
 * 				(in this example, his private home page is provided through a dynamic site template).
 * 				A user that's not a member of any group will be redirected to the first applicable page of the following:
 * 					* login.landing.page
 * 					* the user's default private profile page     (default.user.private.layout.friendly.url)
 * 					* the first community's default private page
 * 					* the first community's default public page.
*/

package com.liferay.portal.events;

import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import com.liferay.portal.kernel.events.Action;
import com.liferay.portal.kernel.events.ActionException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.struts.LastPath;
import com.liferay.portal.kernel.util.PropsKeys;
import com.liferay.portal.kernel.util.PropsUtil;
import com.liferay.portal.kernel.util.StringPool;
import com.liferay.portal.kernel.util.WebKeys;
import com.liferay.portal.model.Group;
import com.liferay.portal.model.Company;
import com.liferay.portal.model.User;
import com.liferay.portal.model.UserGroup;
import com.liferay.portal.service.UserLocalServiceUtil;
import com.liferay.portal.service.UserGroupLocalServiceUtil;
import com.liferay.portal.service.CompanyLocalServiceUtil;
import com.liferay.portal.util.PortalUtil;

public class CustomLoginPostAction extends Action {
	User user;
	HttpSession session;
	HttpServletRequest request;
	
	/**
	 *  Do a string replacement while catching all Exceptions.
	 *  A String will always be returned, in case of errors either a blank string or the original string x.
	 */
	private String safeStringReplace(String x, String pattern, String replacement) {
		if( x == null ) return "";
		if( pattern == null ) return x;
		if( replacement == null ) replacement = "";
		try {
			return x.replace(pattern, replacement);
		} catch (Exception e) {
			return x;
		}
	}
	
	/**
	 * Do string replacements with Liferay property values.
	 * This method searches for any pattern of the form {any.property} and 
	 * replaces it with the value of the property. If the property does not exist,
	 * the pattern is deleted from the string.
	 * @param x The String in which to make replacements
	 * @return A String with replaced patterns or the original String 
	 */
	private String replaceWithProperties(String x) {
		if( x == null ) return "";
		Pattern p = Pattern.compile("\\{.+\\..+\\}");
		Matcher m = p.matcher(new String(x));
		while( m.find() ) {
			String match = m.group();
			String property = match.substring(1, match.length()-1).trim();
			x = safeStringReplace(x, match, PropsUtil.get(property));
		}
		return x;
	}
	
	/**
	 * Do string replacements with Liferay User Data.
	 * This method searches for the pattern {descriptor:identificator} and
	 * replaces it with the value of the found User Data.
	 * At the moment only {UserGroupId:user-group-name} is supported.
	 * @param x The String in which to make replacements
	 * @return A String with replaced patterns or the original String 
	 */
	private String replaceWithData(String x) {
		if( x == null ) return "";
		Pattern p = Pattern.compile("\\{(.+?):(.+?)\\}");
		Matcher m = p.matcher(x);
		while( m.find() ) {
			if( m.groupCount() == 2 ) {
				String completeMatch = m.group(0);
				String dataDescriptor = m.group(1).trim();
				String dataIdentificator = m.group(2).trim();
				String dataResult = "";

				// check for different dataDescriptor Strings and set dataResult to the query result 
				if( dataDescriptor.equals("UserGroupId") ) {
					try {
						dataResult = String.valueOf(
							UserGroupLocalServiceUtil.getUserGroup(this.user.getCompanyId(), dataIdentificator)
								.getUserGroupId()
						);
					} catch ( Exception e ) {}
					if( "".equals(dataResult) ) {
						try {
							for( Company company : CompanyLocalServiceUtil.getCompanies() ) {
								try {
									dataResult = String.valueOf(
										UserGroupLocalServiceUtil.getUserGroup(company.getCompanyId(), dataIdentificator)
											.getUserGroupId()
									);
								} catch ( Exception e ) {}
							}
						} catch ( Exception e ) {}
					}
				} else if( dataDescriptor.equals("Example") ) {
					dataResult = dataIdentificator;
				}

				// use completeMatch and dataResult to do the actual replacement
				x = safeStringReplace(x, completeMatch, dataResult);
			}
		}
		return x;
	}

	/**
	 * Will be fired for each Login Event and redirects the user to her or his landing page.
	 */
	public void run(HttpServletRequest request, HttpServletResponse response)
			throws ActionException {

		try {
			debug("CustomLoginPostAction for User: " + request.getRemoteUser());

			this.request = request;
			this.session = request.getSession();
			this.user = UserLocalServiceUtil.getUser(PortalUtil.getUserId(request));
			LastPath lastPath = null;
			Group firstUserGroup = null;
			
			// First, check for the existence of login.landing.page and login.landing.page.by.group
			// and use the values as pattern where some string replacements are being done
			
			String redirectTo = PropsUtil.get("login.landing.page");
			if(redirectTo != null) debug("Found pattern property login.landing.page");
			
			debug("The user's groups: ");
			for( UserGroup group : user.getUserGroups() ) {
				debug(" - " + group.getName() ); 
			}

			String[] userGroupMapping = PropsUtil.getArray("login.landing.page.by.group");
			if( userGroupMapping != null && userGroupMapping.length > 0 ) {
				debug("Found pattern login.landing.page.by.group");
				for( String userGroupString : userGroupMapping ) {
					debug("Checking if applicable: " + userGroupString);
					if( userGroupString.indexOf(':') > 0 ) {
						String userGroupName = userGroupString.substring(0, userGroupString.indexOf(':')).trim();
						boolean userHasGroup = false;
						for( UserGroup g : user.getUserGroups() ) {
							if( g.getName().equals(userGroupName) ) 
								userHasGroup = true; 
						}
						if( userHasGroup ) {
							redirectTo = userGroupString.substring(userGroupString.indexOf(':')+1).trim();
							debug("User is in UserGroup: " + userGroupName + " -- I'm disregarding login.landing.page and any previous group redirections!");
						}
					}
				}
			}

			// do string replacements
			try {
				debug("Making replacements in redirection string: " + redirectTo);
				redirectTo = safeStringReplace(redirectTo, "{ScreenName}", user.getScreenName());
				redirectTo = safeStringReplace(redirectTo, "{UserId}", String.valueOf(user.getGroup().getGroupId()));
				try {
					redirectTo = safeStringReplace(redirectTo, "{FirstUserGroupId}", String.valueOf(user.getUserGroups().get(0).getUserGroupId()));
				} catch( IndexOutOfBoundsException e ) {}
				redirectTo = safeStringReplace(redirectTo, "{UserURLGroup}", user.getGroup().getFriendlyURL());
				redirectTo = safeStringReplace(redirectTo, "{PrivateUserMapping}", 
						PropsUtil.get(PropsKeys.LAYOUT_FRIENDLY_URL_PRIVATE_USER_SERVLET_MAPPING));
				redirectTo = safeStringReplace(redirectTo, "{PrivateGroupMapping}", 
						PropsUtil.get(PropsKeys.LAYOUT_FRIENDLY_URL_PRIVATE_GROUP_SERVLET_MAPPING));
				redirectTo = safeStringReplace(redirectTo, "{PublicMapping}", 
						PropsUtil.get(PropsKeys.LAYOUT_FRIENDLY_URL_PUBLIC_SERVLET_MAPPING));
				redirectTo = safeStringReplace(redirectTo, "{UserDefaultPrivateLayout}", 
						PropsUtil.get(PropsKeys.DEFAULT_USER_PRIVATE_LAYOUT_FRIENDLY_URL));
				redirectTo = safeStringReplace(redirectTo, "{UserDefaultPublicLayout}", 
						PropsUtil.get(PropsKeys.DEFAULT_USER_PUBLIC_LAYOUT_FRIENDLY_URL));
				redirectTo = replaceWithData(redirectTo);				
				redirectTo = replaceWithProperties(redirectTo);
			}
			catch (Exception e) {}
			
			if( redirectTo != null && redirectTo.length() > 0) {
				debug("Redirecting to: " + redirectTo);
				lastPath = new LastPath(StringPool.BLANK, redirectTo);
			}

			// Second, look for User's Private Pages
			if(lastPath == null && user.hasPrivateLayouts()) {
				debug("Looking for User's Private Pages");
				String friendlyURL = PropsUtil.get(PropsKeys.LAYOUT_FRIENDLY_URL_PRIVATE_USER_SERVLET_MAPPING)
						+ user.getGroup().getFriendlyURL()
						+ PropsUtil.get(PropsKeys.DEFAULT_USER_PRIVATE_LAYOUT_FRIENDLY_URL);
				try {
					debug("Checking if URL is valid: " + friendlyURL);
					long plid = PortalUtil.getPlidFromFriendlyURL(user.getCompanyId(), friendlyURL);
					debug("Got Plid for Layout: " + plid);
					if( plid > 0 ) {
						lastPath = new LastPath(StringPool.BLANK, friendlyURL);
						debug("Redirecting to User's Private Pages");
					}
				} catch( Exception e ) {
					// layout does not exist => lastPath stays null
					lastPath = null;
					debug("Exception caught: " + e.getMessage());
				}
			}

			if(lastPath == null) {
				// Look for Communities
				debug("Looking for associated Communities");
				List<Group> groups = user.getGroups();
				for(Group group : groups) {
					debug("Community: " + group.getName() + ", friendlyUrl: "
							+ group.getFriendlyURL());
					if(firstUserGroup == null)
						firstUserGroup = group;
						if(group.hasPrivateLayouts()) {
							debug("Redirecting to Organization's Private Pages");
							lastPath = new LastPath(StringPool.BLANK, PropsUtil.get(PropsKeys.LAYOUT_FRIENDLY_URL_PRIVATE_GROUP_SERVLET_MAPPING)+group.getFriendlyURL());
						}
						if(lastPath != null)
							break;
				}
			}

			if(lastPath == null && firstUserGroup != null) {
				// Redirect to the Public Pages of the first found
				// Community/Organization
				debug("Redirecting to Community/Organization's Public Pages (fallback option)");
				lastPath = new LastPath(StringPool.BLANK, PropsUtil.get(PropsKeys.LAYOUT_FRIENDLY_URL_PUBLIC_SERVLET_MAPPING)+firstUserGroup.getFriendlyURL());
			}

			if(lastPath != null) {
				debug("lastPath = " + lastPath.toString());
				session.setAttribute(WebKeys.LAST_PATH, lastPath);
			}
		} catch (Exception e) {
			throw new ActionException(e);
		}
	}

	/**
	 * Helper Method to create debug log messages
	 * @param msg Log message as String
	 */
	private void debug(String msg) {
		if (_log.isDebugEnabled()) {
			_log.debug("CustomLoginPostAction: " + msg);
		}
		//uncomment this if you want debugging info on tomcat/logs/catalina.out
		//System.out.println("CustomLoginPostAction: " + msg);
	}

	private static Log _log = LogFactoryUtil
			.getLog(CustomLoginPostAction.class);
}
